// file: DrawingPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

/**
 * DrawingPanel is a canvas to draw the grid path as the result of grid match
 *
 * hierarchy: JPanel -> DrawingPanel
 */
public class DrawingPanel extends JPanel {

    //**********************************************************************
    //
    // declare global variables
    //
    //**********************************************************************

    
    // the array to store the path to be drawn
    //
    private int data[];

    // the length of the path
    //
    private int datalength;

    // size of the canvas
    //
    private int panelwidth;
    private int panelheight;

    // reserved margin at left and bottom of the panel
    //
    private int xMargin;
    private int yMargin;

    // the range of each dimension
    //
    private int xMax = 32;
    private int xMin = 0;
    private int yMax = 32;
    private int yMin = 0; 

    // variable to control the draw process
    //
    public boolean dataAttached;

    // the main canvas to paint
    //
    private Graphics graph;

    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * default class constructor
     */
    public DrawingPanel() {
	super();
	setBackground(Color.white);
	datalength = 0;
	xMargin = 25;
	yMargin = 20;
	xMax = 32;
	xMin = 0; 
	yMax = 32;
	yMin = 0; 

	// initiate the flag used to control the drawing process
	dataAttached = false;
    }

    // *****************************************************************
    // 
    // declare class methods
    //
    // *****************************************************************

    /**
     * the major draw method
     * clear the canvas, draw the grid, draw the path if data attached
     *
     * @param g Graphics object to use
     */
    public void paint(Graphics g) {
	super.paint(g);
	graph = g;

	// clear panel before paint so that image would not be overlapped
	clearPanel();

	// draw grid lines first
	DrawGrid();

	// draw the path if data attched
	if (dataAttached) {
	    DrawData(xMax, xMin);
	}
    }

    // method: DrawGrid
    //
    // arguments: none
    // return   : none
    //
    // draw the grid lines according to the size of canvas
    //
    private void DrawGrid() {
	int i;

	// get the parameters about the canvas size
	DetermineDimensions();

	// set step length for each dimension
	if(xMax == xMin) xMax = xMin+1;
	if(yMax == yMin) yMax = yMin+1;

	double xStep = (double)(panelwidth - 2*xMargin) / (xMax - xMin);
	double yStep = (double)(panelheight - 2*yMargin) / (yMax - yMin);

	
	double xGridDiv = (int)(20.0/xStep);
	if(xGridDiv == 0.0)
	    xGridDiv = xStep;
	else
	    xGridDiv = xGridDiv * xStep;

	double yGridDiv = (int)(20.0/yStep);
	if(yGridDiv == 0.0)
	    yGridDiv = yStep;
	else
	    yGridDiv = yGridDiv * yStep;
	// grid lines will be drawn in lighter color
	graph.setColor(Color.lightGray);

	int pos;

	// draw grid line for x direction
	/*
	for (i=(panelheight-yMargin); 
	     i>=yMargin;
	     i=(int)(i-yGridDiv)) {
	    graph.drawLine(xMargin, i, panelwidth-xMargin, i);
	}
     */
	for(i=0;
	    (pos=panelheight-yMargin-(int)(i*yGridDiv))>=yMargin;
	    i++) {
	    graph.drawLine(xMargin, pos,
			   panelwidth-xMargin,pos);
	}



	// draw grid line for y direction
	/*
	for (i=xMargin; i<(panelwidth-xMargin); i=(int)(i+xGridDiv)) {
	    graph.drawLine(i, (panelheight-yMargin), 
			   i, yMargin);
	}
	*/

	for(i=0;
	    (pos=xMargin+(int)(i*xGridDiv))<=panelwidth-xMargin;
	    i++) {
	    graph.drawLine(pos, panelheight-yMargin,
			   pos, yMargin);
	}
	
	// frame of the grid plane will be drawn in darker color
	graph.setColor(Color.gray);
	// draw the frame line for the canvas
	graph.drawLine(xMargin, panelheight-yMargin,
		       panelwidth - xMargin, panelheight-yMargin);
	graph.drawLine(xMargin, yMargin, 
		       xMargin, panelheight-yMargin);
	graph.drawLine(xMargin, yMargin,
		       panelwidth - xMargin, yMargin);
	graph.drawLine(panelwidth - xMargin, panelheight-yMargin, 
		       panelwidth - xMargin, yMargin);

	// write the max and min of x and y
	graph.setColor(Color.black);

	String xmin = new String(new Integer(xMin).toString());
	String xmax = new String(new Integer(xMax).toString());
	String ymin = new String(new Integer(yMin).toString());
	String ymax = new String(new Integer(yMax).toString());

	graph.drawString(xmin, xMargin-2, panelheight-5);
	graph.drawString(xmax, panelwidth-xMargin-18,
			 panelheight-5);
	graph.drawString(ymin, 3, panelheight-yMargin);
	graph.drawString(ymax, 3, yMargin + 10);

	if(dataAttached) {
	    // variables to store the coordinates of adjacent points
	    int x1, x2, y1, y2;
	    // for adjacent points (x1, y1) and (x2, y2), draw a line to connect 
	    // them to form a path. path will be drawn in blue
	    graph.setColor(Color.blue);
	    
	    x1 = xMargin;
	    y1 = panelheight - yMargin;
	    
	    for (i=xMin+1; i<=xMax; i++) {
		x2 = (int)(xMargin + (i-xMin)*xStep);
		y2 = (int)(panelheight - yMargin - 
		    (data[i] - yMin) * yStep);
		graph.drawLine(x1, y1, x2, y2);
		x1 = x2;
		y1 = y2;
	    }
	    
	    
	}
    }
   
    /**
     * find out the size of current canvas, assign value to panelwidth and
     * panelheight
     */
    public void DetermineDimensions() {
	Dimension dimen = this.getSize();
	panelwidth = dimen.width;
	panelheight = dimen.height;
    }

    // method: attachData
    //
    // 
    // return   : none
    //
    /**
     * accept the path d[], copy to data[]. also copy the length of path 
     * to datalength. set dataAttached to true.
     *
     * @param d   data to be used
     * @param len length of data
     */
    public void attachData(int d[], int len) {
	data = d;
	datalength = len;

	xMax = datalength-1;
	yMax = data[xMax];
	xMin = 0;
	yMin = data[xMin];

	dataAttached = true;
    }

    /**
     * Method assigns null to data variable, 0 to datalenghth, and false to 
     * dataAttached flag
     */
    public void detachData() {
	data = null;
	datalength = 0;
	dataAttached = false;
    }

    // method: DrawData
    //
    // arguments: path range 
    // return   : none
    //
    // draw the path in the range of min x and max x
    // 
    private void DrawData(int maxx, int minx) {
    }

    /**
     * clear the whole canvas
     */
    public void clearPanel() {
	graph.clearRect(0, 0, panelwidth, panelheight);
    }

    /**
     * draw the original path after reset
     */
    public void recoverPanel() {
	if (data != null) {
	    xMax = datalength-1;
	    xMin = 0;
	    yMax = data[xMax];
	    yMin = 0;
	}
    }

    /**
     * set the max of x to the parameter
     *
     * @param a integer value to set
     */
    public void setXMax(int a) {
	xMax = a;
    }

    /**
     * set the min of x to the parameter
     *
     * @param a integer value to set
     */
    public void setXMin(int a) {
	xMin = a;
    }

    /**
     * set the max of y to the parameter
     *
     * @param a integer value to set
     */
      public void setYMax(int a) {
	yMax = a;
    }

    public void updateYMax() {
	if (data != null) 
	    yMax = data[xMax];
    }

    /**
     * set the min of y to the parameter
     *
     * @param a integer value to set
     */
    public void setYMin(int a) {
	yMin = a;
    }

    /**
     * sets the yMin variable to data[xMin] value
     */
    public void updateYMin() {
	if (data != null) 
	    yMin = data[xMin];
    }

    /**
     * access function of xMax
     *
     * @return xMax value
     */
    public int getXMax() {
	return xMax;
    }

    /**
     * access function of  xMin
     *
     * @return xMin value
     */
    public int getXMin() {
	return xMin;
    }

    /**
     * access function of yMax
     *
     * @return yMax value
     */
    public int getYMax() {
	return yMax;
    }

    /**
     * access function of yMin
     *
     * @return yMin value
     */
    public int getYMin() {
	return yMin;
    }

    /**
     * access function of xMargin
     *
     * @return xMargin
     */
    public int getXMargin() {
	return xMargin;
    }

    /**
     * access function of yMargin
     *
     * @return yMargin value
     */
    public int getYMargin() {
	return yMargin;
    }

}

// end of file
