// file: EnergyPlot.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

/**
 * EnergyPlot is a canvas to draw the energy plot of a lpc signal.
 *
 * hierarchy: JPanel -> SubPanel -> EnergyPlot
 */
public class EnergyPlot extends SubPanel {

    // *******************************************************************
    //
    // declare global variables
    //
    // *******************************************************************

    // the lpc data of the signal to be drawn
    private lpcData signal;

    // the array to store the energy
    private int energy[];

    // signal range of each dimension
    private int xMax = 32;
    private int xMin = 0;
    private int yMax = 10;
    private int yMin = 0;

    // the main canvas to paint
    private Graphics graph;

    // the mode this plot will be drawn
    private String drawMode = null;

    // ******************************************************************
    //
    // declare class constructor
    //
    // ******************************************************************
    /**
     * default class constructor
     */
    public EnergyPlot() {
	super();
	setBackground(Color.white);

	// inititate the drawing process control and energy array
	signal = null;
	energy = null;
    }

    // ******************************************************************
    //
    // declare class methods
    //
    // ******************************************************************

    /**
     * the major draw method
     * clear the canvas, draw the grid, compute the energy of lpc coded signal
     * and draw the energy signal if signal attached
     *
     * @param g Graphics object to be used
     */
    public void paint(Graphics g) {
	super.paint(g);
	graph = g;
	// clear panel before painting so that image would not be overlapped
	clearPanel();

	// draw the energy plot horizontally or vertically according to the
	// drawing mode
	if (signal == null) {
	    return;
	}
	else if (drawMode == "Horizontal") {
	    drawHorizontal();
	}
	else if (drawMode == "Vertical") {
	    drawVertical();
	}
    }

    // method: drawHorizontal
    //
    // arguments: none
    // return   : none
    //
    // draw the energy plot horizontally
    //
    private void drawHorizontal() {
	int i;
	// variables to store the coordinates of adjacent points
	int x1, x2, y1, y2;

	// set the margin at bottom
	int yMargin = 5;
	int xMargin = 25;

	Dimension dim = this.getSize();
	int panelwidth = dim.width;
	int panelheight = dim.height;

	if (yMax == yMin) {
	    yMax = yMin + 1;
	}

	int yStep = (panelheight - 2*yMargin) / (yMax - yMin);
	
	graph.setColor(Color.gray);
	graph.drawLine(xMargin, yMargin,
		       xMargin, panelheight - yMargin);
	graph.drawLine(xMargin, panelheight - yMargin,
		       panelwidth - xMargin, panelheight - yMargin);
	
	graph.setColor(Color.blue);
	x1 = xMargin;
	y1 = panelheight - yMargin;
	for (i=xMin+1; i<=xMax; i++) {
	    x2 = xMargin + (int)((i-xMin)*(panelwidth-2*xMargin)/(xMax-xMin));
	    y2 = panelheight - yMargin - (energy[i] - yMin) * yStep;
	    graph.drawLine(x1,y1, x2, y2);
	    x1 = x2;
	    y1 = y2;
	}

    }


    // method: drawVertical
    //
    // arguments: none
    // return   : none
    //
    // draw the energy plot vertically
    //
    private void drawVertical() {
	int i;
	// variables to store the coordinates of adjacent points
	int x1, x2, y1, y2;

	int xMargin = 5;
	int yMargin = 20;

	Dimension dim = this.getSize();
	int panelwidth = dim.width;
	int panelheight = dim.height;

	if (yMax == yMin) {
	    yMax = yMin + 1;
	}

	int yStep = (panelheight - 2*yMargin) / (yMax - yMin);

	graph.setColor(Color.gray);
	graph.drawLine(panelwidth-xMargin, panelheight-yMargin,
		       xMargin, panelheight - yMargin);
	graph.drawLine(panelwidth-xMargin, panelheight - yMargin,
		       panelwidth - xMargin, yMargin);
	
	graph.setColor(Color.blue);

	y1 = panelheight - yMargin;
	x1 = panelwidth -xMargin;

	for (i=xMin+1; i<=xMax; i++) {
	    y2 = panelheight-yMargin - (int)((i-xMin)*(panelheight-2*yMargin)/(xMax-xMin));
	    x2 = panelwidth - xMargin - (energy[i] - yMin) * yStep;
	    graph.drawLine(x1,y1, x2, y2);
	    x1 = x2;
	    y1 = y2;
	}
    }	

    /**
     * accept lpc coded signal, compute energy for each frame and store the
     * value in data[]
     *
     * @param lpc lpcData object to be attached
     */
    public void attachData(lpcData lpc) {
	if (lpc == null) {
	    signal = null;
	    repaint();
	    return;
	}
	signal = lpc;

	xMax = signal.getLength() - 1;
	xMin = 0;

	energy = new int[xMax+1];
	// compute energy for each frame and store value in data[]
	for (int i=0; i<=xMax; i++) {
	    energy[i] = (int)computeEnergy(signal.data[i]);
	}
    }

    // method: computeEnergy
    //
    // arguments: double ar[]
    // return   : double
    //
    // accept one frame of lpc vector, compute and return the energy
    //
    private double computeEnergy(double ar[]) {
	int i, j, k;
	double km, d;
	double rr[] = new double[13];
	double rf[] = new double[13];
	double rf1[] = new double[13];
	double gain;

	// lpcar2rf --- calculate {km}
	// convert autoregressive coeff. ar[] to reflection coeff. rf[]
	for (i=1; i<13; i++) {
	    rf[i] = ar[i];
	}

	// double w = 1;
	for (j=11; j>=1; j--) {
	    km = rf[j+1];
	    d = 1.0/(1.0-km*km);
	    for (i=1; i<=j; i++) {
		rf1[i] = (rf[i] - km * rf[j-i+1]) * d;
	    }
	    for (i=1; i<=j; i++) {
		rf[i] = rf1[i];
	    }
	}
	
	// lpcrf2rr --- calculate {r}, r[0] = E0
	// convert reflection coeff. rf[] to autocorrelation coeff. rr[]
	double a[] = new double[13];
	double sum;
	for(i=0;i<13;i++) {
	    rr[i] = 0.0;
	    a[i] = 0.0;
	}
	a[0] = rf[1];
	rr[0] = 1.0;
	rr[1] = - a[0];
	double e = a[0]*a[0] - 1.0;

	for(i=1;i<12;i++) {
	    km = rf[i+1];
	    sum = 0.0;
	    for(j=i;j>=1;j--)
		sum += rr[j]*a[i-j];

	    rr[i+1] = km*e - sum;

	    double aa[] = new double[13];
	    for(j=0;j<i;j++) 
		aa[j] = a[j] + km*a[i-j-1];
	    for(j=0;j<i;j++)
		a[j] = aa[j];
	    a[i] = km;

	    e = e*(1.0 - km*km);
	}

	ar[0] = 1.0;
	for(i=0;i<12;i++)
	    ar[i+1]=a[i];

	sum = 0.0;
	for(i=0;i<13;i++) 
	    sum += rr[i]*ar[i];

	double r0 = 1.0 / sum;

	for(i=0;i<13;i++)
	    rr[i] *= r0;

	if (rr[0] > yMax) 
	    yMax = (int)rr[0];

	return rr[0];
    }

    /**
     * clear canvas
     */
    public void clearPanel() {
	graph.clearRect(0, 0, this.getSize().width, this.getSize().height);
    }

    /**
     * set display range to full range
     */
    public void recoverPanel() {
	xMax = signal.getLength() - 1;
	xMin = 0;
	//	yMax = 10;
	//	yMin = 0;
	repaint();
    }

    /**
     * set display range for max of x
     *
     * @param a integer value to set xMax
     */
    public void setXMax(int a) {
	xMax = a;
    }

    /**
     * set display range for min of x
     *
     * @param a integer value to set xMin
     */
    public void setXMin(int a) {
	xMin = a;
    }

    /**
     * set the drawing mode
     * 
     * @param a a = "Horizontal" to draw energy plot horizontally.
     *          a = "Vertical" to draw energy plot Vertically.
     *          otherwise, will not draw
     */
    public void setDrawMode(String a) {
	drawMode = a;
    }
}

// end of file
