// file: SpectrumPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.lang.*;
import java.net.*;
import java.awt.event.*;

/**
 * SpectrumPanel is a canvas to draw the spectrogram for attached signal
 * each frequency interval of each frame will be painted as a color cell
 * connect the color cells together to form a spectrogram.
 *
 * hierarchy: JPanel -> SpectrumPanel
 */
public class SpectrumPanel extends JPanel {

    //**********************************************************************
    //
    // declare global variables
    //
    //**********************************************************************

    // lpc coded signal to be drawn
    //
    private lpcData signal;

    // number of frequency intervals
    //
    private int num_of_freq = 256;

    // path info for the warped signal
    //
    private int path[];
    
    // display area
    //
    private int xMax;
    private int xMin;

    // array to store the spectrogram
    //
    private double energy[][];

    // max value of energy[][]
    //
    private double energyMax = 0.0;

    // array to store the index from colormap for each color cell
    //
    private int colorindex[][];

    // main canvas
    //
    private Graphics graph;

    // flag to distinguish the painting mode
    //
    private String drawMode;

    //******************************************************************
    //
    // declare class constructor
    //
    //******************************************************************

    /**
     * constructor initializes variables and components
     */
    public SpectrumPanel() {
	super();
	setBackground(Color.white);
	signal = null;
	path = null;
	drawMode = null;
    }

    // *****************************************************************
    // 
    // declare class methods
    //
    // *****************************************************************

    /** 
     * the main paint method. 
     * get size of current canvas, clear canvas, paint spectrogram 
     * according the drawMode
     *
     * @param g Graphics object in use
     */
    public void paint(Graphics g) {

	graph = g;

	// clear canvas before painting 
	clearPanel();

	if (signal == null) {
	    return;
	}
	else if (drawMode == "Horizontal") {
	    drawHorizontal();
	}
	else if (drawMode == "Vertical") {
	    drawVertical();
	}
	else if (drawMode == "Warped") {
	    drawWarped();
	}    
    }

    /**
     * clear the canvas
     */
    public void clearPanel() {
	graph.clearRect(0, 0, this.getWidth(), this.getHeight());
    }

    /**
     * draw spectrogram in full range after reset
     */
    public void recoverPanel() {
	xMax = signal.getLength()-1;
	xMin = 0;
	repaint();
    }

    /**
     * to draw the spectrogram vertically
     */
    private void drawVertical() {
	int i, j, x, y;
	int xMargin = 5, yMargin = 20;

	Dimension dim = this.getSize();
	int width = dim.width;
	int height = dim.height;

	for (i=width - xMargin; i>xMargin; i--) {
	    for (j=height - yMargin; j>yMargin; j--) {
		x = (int)((width-xMargin-i)*num_of_freq*0.5/(width-2*xMargin));
		y = (int)((height-yMargin-j)*(xMax-xMin)/(height-2*yMargin));
		graph.setColor(Constants.colortable[colorindex[y][x]]);
		graph.fillRect(i,j,1,1);
	    }
	}

    }

    /**
     * to draw the spectrogram horizontally
     */
    public void drawHorizontal() {
	int i, j, x, y;
	int xMargin = 25, yMargin = 5;

	Dimension dim = this.getSize();
	int width = dim.width;
	int height = dim.height;

	for (i=xMargin; i<width-xMargin; i++) {
	    for (j=yMargin; j<height-yMargin; j++) {
		x = (int)((i-xMargin)*(xMax-xMin)/(width-2*xMargin));
		y = (int)((j-yMargin)*num_of_freq*0.5/(height-2*yMargin));
		graph.setColor(Constants.colortable[colorindex[x][y]]);
		graph.fillRect(i,height-j,1,1);
	    }
	}
    }

    /**
     * to draw the spectrogram for warped template according to the 
     * matching path 
     */
    public void drawWarped() {
	int i, j, x, y;
	int xMargin = 25, yMargin = 5;

	Dimension dim = this.getSize();
	int width = dim.width;
	int height = dim.height;

	for (i=xMargin; i<width-xMargin; i++) {
	    for (j=yMargin; j<height-yMargin; j++) {
		x = (int)((i-xMargin)*(xMax-xMin)/(width-2*xMargin));
		y = (int)((j-yMargin)*num_of_freq*0.5/(height-2*yMargin));
		graph.setColor(Constants.colortable[colorindex[path[x]][y]]);
		graph.fillRect(i,height-j,1,1);
	    }
	}
	      
    }

    /**
     * set signal and relative parameters, compute spectrogram
     *
     * @param d lpcData of speech signal to be used
     */
    public void attachData(lpcData d) {
	if (d == null) {
	    signal = null;
	    repaint();
	    return;
	}
	signal = d;
	xMax = signal.getLength() - 1;
	xMin = 0;
	energy = new double[signal.getLength()][num_of_freq/2];
	colorindex = new int[signal.getLength()][num_of_freq/2];
	computeSpectrum();
    }


    /**
     * send warping path to canvas to draw spectrogram for warped signal
     *
     * @param d   array for path info
     * @param len length of data path
     */
    public void attachPath(int d[], int len) {
	path = d;
	xMax = len-1;
	xMin = 0;
    }
	
    /**
     * compute the spectrogram for the signal, fill in the energy array
     */
    private  void computeSpectrum() {
	int i, j, k;

	// real part of spectrum
	//
	double real=0;

	// virtual part of spectrum
	//
	double virtual=0;

	// max value of energy at one frame
	//
	double max = 5.0;

	// array to store gain for each frame
	//
	double G[] = new double[signal.getLength()];

	//	max = 0;
	// compute gain for each frame
	//
	for (i=0; i<signal.getLength(); i++) {
	    G[i] = computeGain(signal.data[i]);
	}

	// normalize gain by the maximum value
	//
	for (i=0; i<signal.getLength(); i++) {
	    G[i] = G[i] / energyMax;
	}
	
	System.out.println("energyMax: " + energyMax);

	// compute energy for each frame at each frequency interval using:
	//                             G
	//          H(z) = --------------------------
	//                  1 - sum ( a_k * z^(-k) ),     k=1,...,12
	//
	for (i=0; i<signal.getLength(); i++) {
	    energy[i][0] = 0;

	    for (j=0; j<num_of_freq/2; j++) {
		real = 0;
		virtual = 0;
		for (k=1; k<13; k++) {

		    // real part of sum( a_k * z^(-k) ):
		    // real = sum[ cos(2*pi*k*j/num_of_freq) ]
		    //
		    real += signal.data[i][k] *
			Math.cos(Math.PI*2*k*j/num_of_freq);

		    // virtual part of sum( a_k * z^(-k) ):
		    // virtual = sum[ sin(2*pi*k*j/num_of_freq) ]
		    //
		    virtual += signal.data[i][k] * 
			Math.sin(Math.PI*2*k*j/num_of_freq);
		}

		//                                 G
		//   energy[i][j] = --------------------------------
		//                   (1 + real)^2 + (1 + virtual)^2
		//
		energy[i][j] = G[i] / ((1+ real)*(1+ real) + 
				  virtual*virtual);
		energy[i][j] = Math.log(energy[i][j]*2 + 1);

		// store maximum value of energy[i][j] in max
		//
		if (energy[i][j] > max) 
		    max = energy[i][j];
	    }
	}

	System.out.println("max of energy: " + max);

	// convert value of energy[i][j] into color index
	//
	for (i=0; i<signal.getLength(); i++) {
	    for (j=0; j<num_of_freq/2; j++) {
		colorindex[i][j] = (int)(255*energy[i][j]/max);
	    }
	}

    }

    /**
     * compute gain for input autoregressive coeff.
     *
     * @param ar array used to compute gain
     */
    private double computeGain(double ar[]) {
	int i, j, k;
	double km, d;
	double rr[] = new double[13];
	double rf[] = new double[13];
	double rf1[] = new double[13];
	double gain;

	// lpcar2rf --- calculate {km}
	// convert autoregressive coeff. ar[] to reflection coeff. rf[]
	//
	for (i=1; i<13; i++) {
	    rf[i] = ar[i];
	}

	// double w = 1;
	//
	for (j=11; j>=1; j--) {
	    km = rf[j+1];
	    d = 1.0/(1.0-km*km);
	    for (i=1; i<=j; i++) {
		rf1[i] = (rf[i] - km * rf[j-i+1]) * d;
	    }
	    for (i=1; i<=j; i++) {
		rf[i] = rf1[i];
	    }
	}
	
	// lpcrf2rr --- calculate {r}, r[0] = E0
	// convert reflection coeff. rf[] to autocorrelation coeff. rr[]
	//
	double a[] = new double[13];
	double sum;
	for(i=0;i<13;i++) {
	    rr[i] = 0.0;
	    a[i] = 0.0;
	}
	a[0] = rf[1];
	rr[0] = 1.0;
	rr[1] = - a[0];
	double e = a[0]*a[0] - 1.0;

	for(i=1;i<12;i++) {
	    km = rf[i+1];
	    sum = 0.0;
	    for(j=i;j>=1;j--)
		sum += rr[j]*a[i-j];

	    rr[i+1] = km*e - sum;

	    double aa[] = new double[13];

	    for(j=0;j<i;j++) 
		aa[j] = a[j] + km*a[i-j-1];
	    for(j=0;j<i;j++)
		a[j] = aa[j];
	    a[i] = km;

	    e = e*(1.0 - km*km);
	}

	ar[0] = 1.0;
	for(i=0;i<12;i++)
	    ar[i+1]=a[i];

	sum = 0.0;
	for(i=0;i<13;i++) 
	    sum += rr[i]*ar[i];

	double r0 = 1.0 / sum;

	for(i=0;i<13;i++)
	    rr[i] *= r0;

	// calculate G
	sum = 1.0;
	for (i=1; i<13; i++) {
	    sum = sum * (1.0 - rf[i] * rf[i]);
	}
	gain = rr[0] * sum;
	if (rr[0] > energyMax) {
	    energyMax = rr[0];
	}
	return gain;
    }

    /**
     * set the drawing mode
     *
     * @param a Horizontal --> draw spectrogram horizontally
     *          Vertical   --> draw spectrogram vertically
     *          Path       --> draw spectrogram horizontally for warped
     *                           template
     */
    public void setDrawMode(String a) {
	drawMode = a;
    }

    /**
     * set maximum value of display range
     *
     * @param a int value to set
     */
    public void setXMax(int a) {
	xMax = a;
    }

    /**
     * set minimum value of display range
     *
     * @param a int value to be set
     */
    public void setXMin(int a) {
	xMin = a;
    }

} 

// end of file
