// file: WaveformPanel.java
//

// import necessary libraries
//
import java.io.*;
import java.net.*;
import java.awt.*;
import javax.swing.*;

/**
 * class WaveformPanel is a canvas to draw the waveform of a speech signal
 * 
 * hierarchy: JPanel -> WaveformPanel
 */
public class WaveformPanel extends JPanel{

    // *******************************************************************
    //
    // declare global variables
    //
    // *******************************************************************

    // array to store the sample value
    //
    private int data[] = new int[10000];

    // length of raw file
    //
    private int length = 0;

    // display range
    //
    private int xMax = 0;
    private int xMin = 0;

    // maximum value of amplitude
    //
    private int amplitude;

    // ******************************************************************
    //
    // declare class constructor
    //
    // ******************************************************************

    /**
     * constructor, just set background to white
     */
    public WaveformPanel() {
	super();
	setBackground(Color.white);
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************
    
    /**
     * open raw file specified by input url, read in sample value and store
     * in data[] then draw the wave form
     *
     * @param url URL signal to set
     */
    public void setSignal(URL url) {
	readData(url);
	repaint();
    }

    /**
     * access method for length
     *
     * @return length of raw file
     */
    public int getLength() {
	return length;
    }

    /**
     * open file, read in sample value and store in data[]
     *
     * @param fileurl URL of file
     */
    private void readData(URL fileurl) {
	int i=0;

	// declare the input stream to access the raw file
	DataInputStream dis = null;

	try {
	    // open the stream
	    dis = new DataInputStream(fileurl.openStream());
	}
	catch (IOException e) {
	    // if can't open the file, set special value to data and length
	    // so that an error message can be checked
	    System.out.println("error: " + e);
	    data = null;
	    length = 0;
	    return;
	}

	// read in each 2-byte integer number as a sample until hit end
	// of file
	while (true) {
	    try {
		data[i++] = dis.readShort();
	    }
	    catch (EOFException e) {
		length = i;
		break;
	    }
	    catch (IOException e) {
		System.out.println("error: " + e);
		data = null;
		length = 0;
		return;
	    }
	}

	// reset display range to full signal length
	xMax = length - 1;
	xMin = 0;
	System.out.println("length of waveform: " + length);
    }

    /**
     * the main paint method for wave form
     * display range is limited by xMax and xMin
     * waveform will be drawn by connecting adjacent samples
     *
     * @param g Graphics object to use
     */
    public void paint(Graphics g) {

	// variables to store location of adjacent samples
	// (x1, y1) is the coordinates for sample at i
	// (x2, y2) is the coordinates for sample at j
	//
	int x1, y1, x2, y2;

	// size of current panel
	//
	int width, height;
	int i;

	// margin to be left for each dimension
	//
	int xMargin = 25, yMargin = 5;

	width = this.getSize().width;
	height = this.getSize().height;

	// clear canvas before draw waveform
	//
	g.clearRect(0,0,width, height);

	if (length == 0) {
	    return;
	}
	
	g.setColor(Color.blue);

	if (xMin == xMax) {
	    xMax = xMin + 1;
	}

	// check y-dimension range
	//
	amplitude = 0;
	for(i=xMin; i< xMax;i++) {
	    if(Math.abs(data[i])>amplitude)
		amplitude = Math.abs(data[i]);
	}
	
	// step length for y-dimension
	//
	double scale=(double)(height/2-yMargin)/amplitude;

	System.out.println("amplitude="+amplitude+"scale="+scale);

	// initialize point (x1, y1)
	//
	x1 = xMargin;
	y1 = (int)(height/2 - data[xMin]*scale);

	// draw waveform
	//
	for (i=xMin+1; i<xMax; i++) {
	    x2 = (int)((i-xMin)*(width-2*xMargin)/(xMax-xMin)) + xMargin;
	    y2 = (int)(height/2 - data[i]*scale);
	    g.drawLine(x1,y1,x2,y2);
	    x1 = x2;
	    y1 = y2;
	}
	
	// draw grid line and amplitude
	//
	g.setColor(Color.gray);
	g.drawLine(xMargin, height/2, width-xMargin, height/2);
	g.drawLine(xMargin, yMargin, xMargin, height-yMargin);

	g.setColor(Color.black);
	g.drawString(new String(new Integer(amplitude).toString()),
			 5, 10);
	g.drawString(new String(new Integer(-amplitude).toString()),
			 1, height-2);
    }

    /**
     * set display range to full range and repaint panel
     */
    public void recoverPanel() {
	xMax = length - 1;
	xMin = 0;
	repaint();
    }

    /**
     * Sets xMax.  
     *
     * sampling rate =8000 per sec
     * frame length = 0.01 sec
     * thus one frame corresponds to 8000*0.01=80 samples
     *
     * @param a value that will be multiplied by 80 then stored as xMax
     */
    public void setXMax(int a) {
	xMax = a * 80;
    }

    /**
     * Sets xMin.  
     *
     * sampling rate =8000 per sec
     * frame length = 0.01 sec
     * thus one frame corresponds to 8000*0.01=80 samples
     *
     * @param a value that will be multiplied by 80 then stored as xMin
     */
    public void setXMin(int a) {
	xMin = a * 80;
    }
}
